@@ -32,9 +32,6 @@
 #include <gst/gstghostpad.h>
 #include <gst/gstutils.h>
 
-QT_BEGIN_HEADER
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 namespace Gstreamer
@@ -105,6 +102,7 @@ inline void AudioDataOutput::convertAndEmit()
 
     for (int i = 0 ; i < m_channels ; ++i) {
         map.insert(static_cast<Phonon::AudioDataOutput::Channel>(i), m_channelBuffers[i]);
+        Q_ASSERT(i == 0 || m_channelBuffers[i - 1].size() == m_channelBuffers[i].size());
     }
 
     emit dataReady(map);
@@ -206,7 +204,4 @@ void AudioDataOutput::processBuffer(GstElement*, GstBuffer* buffer, GstPad*, gpo
 
 }} //namespace Phonon::Gstreamer
 
-QT_END_NAMESPACE
-QT_END_HEADER
-
 #include "moc_audiodataoutput.cpp"
