@@ -1,7 +1,7 @@
 /*  This file is part of the KDE project.
 
     Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-    Copyright (C) 2011 Trever Fischer <tdfischer@fedoraproject.org>
+    Copyright (C) 2011 Torrie Fischer <tdfischer@kde.org>
     Copyright (C) 2011 Harald Sitter <sitter@kde.org>
 
     This library is free software: you can redistribute it and/or modify
@@ -17,13 +17,13 @@
     along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "mediaobject.h"
+
 #include <cmath>
 #include <gst/interfaces/navigation.h>
 #include <gst/interfaces/propertyprobe.h>
-#include "mediaobject.h"
 #include "backend.h"
 #include "streamreader.h"
-#include "phonon-config-gstreamer.h"
 #include "debug.h"
 #include "gsthelper.h"
 #include "pipeline.h"
@@ -36,7 +36,7 @@
 #include <QtCore/QStringList>
 #include <QtCore/QTimer>
 #include <QtCore/QVector>
-#include <QtGui/QApplication>
+#include <QApplication>
 #include <QtGui/QFont>
 #include <phonon/GlobalDescriptionContainer>
 
@@ -44,8 +44,6 @@
 #define MAX_QUEUE_TIME 20 * GST_SECOND
 #define GST_PLAY_FLAG_TEXT (1 << 2)
 
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 namespace Gstreamer
@@ -88,10 +86,11 @@ MediaObject::MediaObject(Backend *backend, QObject *parent)
     static int count = 0;
     m_name = "MediaObject" + QString::number(count++);
 
+    m_isValid = true;
     m_root = this;
     m_pipeline = new Pipeline(this);
-    m_isValid = true;
     GlobalSubtitles::instance()->register_(this);
+    GlobalAudioChannels::instance()->register_(this);
 
     connect(m_pipeline, SIGNAL(aboutToFinish()),
             this, SLOT(handleAboutToFinish()), Qt::DirectConnection);
@@ -120,6 +119,8 @@ MediaObject::MediaObject(Backend *backend, QObject *parent)
 
     connect(m_pipeline, SIGNAL(textTagChanged(int)),
             this, SLOT(getSubtitleInfo(int)));
+    connect(m_pipeline, SIGNAL(audioTagChanged(int)),
+            this, SLOT(getAudioChannelInfo(int)));
     connect(m_pipeline, SIGNAL(trackCountChanged(int)),
             this, SLOT(handleTrackCountChange(int)));
 
@@ -129,12 +130,10 @@ MediaObject::MediaObject(Backend *backend, QObject *parent)
 MediaObject::~MediaObject()
 {
     if (m_pipeline) {
-        GstBus *bus = gst_pipeline_get_bus(GST_PIPELINE(m_pipeline->element()));
-        g_signal_handlers_disconnect_matched(bus, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, this);
-        gst_object_unref(bus);
         delete m_pipeline;
     }
     GlobalSubtitles::instance()->unregister_(this);
+    GlobalAudioChannels::instance()->unregister_(this);
 }
 
 void MediaObject::saveState()
@@ -299,7 +298,7 @@ void MediaObject::changeSubUri(const Mrl &mrl)
 
     if (customFont.isNull()) {
         QFont videoWidgetFont = QApplication::font("VideoWidget");
-        fontDesc = videoWidgetFont.family() + " " + QString::number(videoWidgetFont.pointSize());
+        fontDesc = videoWidgetFont.family() + ' ' + QString::number(videoWidgetFont.pointSize());
     }
     //FIXME: Try to detect common encodings, like libvlc does
     g_object_set(G_OBJECT(m_pipeline->element()), "suburi", mrl.toEncoded().constData(),
@@ -319,10 +318,10 @@ void MediaObject::autoDetectSubtitle()
 
         // Remove the file extension
         QString absCompleteBaseName = m_source.fileName();
-        absCompleteBaseName.replace(QFileInfo(absCompleteBaseName).suffix(), "");
+        absCompleteBaseName.replace(QFileInfo(absCompleteBaseName).suffix(), QChar());
 
         // Looking for a subtitle in the same directory and matching the same name
-        foreach(QLatin1String ext, exts) {
+        foreach(const QLatin1String &ext, exts) {
             if (QFile::exists(absCompleteBaseName + ext)) {
                 changeSubUri(Mrl("file://" + absCompleteBaseName + ext));
                 break;
@@ -396,6 +395,36 @@ void MediaObject::loadingComplete()
     link();
 }
 
+void MediaObject::getAudioChannelInfo(int stream)
+{
+    gint channelCount = 0;
+    g_object_get(G_OBJECT(m_pipeline->element()), "n-audio", &channelCount, NULL);
+    if (channelCount)
+        GlobalAudioChannels::instance()->add(this, -1, tr("Default"), "");
+    for (gint i = 0; i < channelCount; ++i) {
+        GstTagList *tags = 0;
+        g_signal_emit_by_name (G_OBJECT(m_pipeline->element()), "get-audio-tags",
+                               i, &tags);
+        if (tags) {
+            gchar *tagLangCode = 0;
+            gchar *tagCodecName = 0;
+            gst_tag_list_get_string (tags, GST_TAG_AUDIO_CODEC, &tagCodecName);
+            gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE, &tagLangCode);
+            QString name;
+            if (tagLangCode)
+                name = QLatin1String(tagLangCode);
+            else
+                name = tr("Unknown");
+            if (tagCodecName)
+                name = QString("%1 [%2]").arg(name, QLatin1String(tagCodecName));
+            GlobalAudioChannels::instance()->add(this, i, name);
+            g_free(tagLangCode);
+            g_free(tagCodecName);
+        }
+    }
+    emit availableAudioChannelsChanged();
+}
+
 void MediaObject::getSubtitleInfo(int stream)
 {
     gint spuCount = 0; // Sub picture units.
@@ -547,7 +576,6 @@ void MediaObject::handleStateChange(GstState oldState, GstState newState)
     if (newState == GST_STATE_READY)
         emit tick(0);
 
-    // Avoid signal emission while processing EOS to avoid bogus UI updates.
     if (!m_doingEOS)
         emit stateChanged(m_state, prevPhononState);
 }
@@ -560,7 +588,8 @@ void MediaObject::handleEndOfStream()
         m_doingEOS = true;
         { // When working on EOS we do not want signals emitted to avoid bogus UI updates.
             emit stateChanged(Phonon::StoppedState, m_state);
-            m_aboutToFinishWait.wakeAll();m_aboutToFinishLock.unlock();
+            m_aboutToFinishWait.wakeAll();
+            m_aboutToFinishLock.unlock();
             m_pipeline->setState(GST_STATE_READY);
             emit finished();
         }
@@ -579,7 +608,7 @@ void MediaObject::handleEndOfStream()
 bool MediaObject::hasInterface(Interface iface) const
 {
     return iface == AddonInterface::TitleInterface || iface == AddonInterface::NavigationInterface
-        || iface == AddonInterface::SubtitleInterface;
+        || iface == AddonInterface::SubtitleInterface || iface == AddonInterface::AudioChannelInterface;
 }
 
 QVariant MediaObject::interfaceCall(Interface iface, int command, const QList<QVariant> &params)
@@ -635,12 +664,47 @@ QVariant MediaObject::interfaceCall(Interface iface, int command, const QList<QV
                     break;
             }
             break;
+        case AudioChannelInterface:
+            switch(command)
+            {
+                case availableAudioChannels:
+                    return QVariant::fromValue(_iface_availableAudioChannels());
+                    break;
+                case currentAudioChannel:
+                    return QVariant::fromValue(_iface_currentAudioChannel());
+                    break;
+                case setCurrentAudioChannel:
+                    if (params.isEmpty() || !params.first().canConvert<AudioChannelDescription>()) {
+                        error() << Q_FUNC_INFO << "arguments invalid";
+                        return QVariant();
+                    }
+                    _iface_setCurrentAudioChannel(params.first().value<AudioChannelDescription>());
+                    break;
+            }
+            break;
         }
     }
     return QVariant();
 }
 #endif
 
+QList<AudioChannelDescription> MediaObject::_iface_availableAudioChannels() const
+{
+    return GlobalAudioChannels::instance()->listFor(this);
+}
+
+AudioChannelDescription MediaObject::_iface_currentAudioChannel() const
+{
+    return m_currentAudioChannel;
+}
+
+void MediaObject::_iface_setCurrentAudioChannel(const AudioChannelDescription &channel)
+{
+    const int localIndex = GlobalAudioChannels::instance()->localIdFor(this, channel.index());
+    g_object_set(G_OBJECT(m_pipeline->element()), "current-audio", localIndex, NULL);
+    m_currentAudioChannel = channel;
+}
+
 QList<MediaController::NavigationMenu> MediaObject::_iface_availableMenus() const
 {
     return m_pipeline->availableMenus();
@@ -681,7 +745,7 @@ void MediaObject::_iface_jumpToMenu(MediaController::NavigationMenu menu)
 
 void MediaObject::handleTrackCountChange(int tracks)
 {
-    m_backend->logMessage(QString("handleTrackCountChange %0").arg(tracks), Backend::Info, this);
+    debug() << Q_FUNC_INFO << tracks;
 
     int old_availableTitles = m_availableTitles;
     m_availableTitles = tracks;
@@ -705,7 +769,7 @@ void MediaObject::_iface_setCurrentTitle(int title)
     if (m_source.discType() == Phonon::NoDisc || title == m_currentTitle) {
         return;
     }
-    m_backend->logMessage(QString("setCurrentTitle %0").arg(title), Backend::Info, this);
+    debug() << Q_FUNC_INFO << title;
     QString format = m_source.discType() == Phonon::Cd ? "track" : "title";
     m_pendingTitle = title;
 
@@ -736,8 +800,8 @@ void MediaObject::_iface_setCurrentSubtitle(const SubtitleDescription &subtitle)
     if (subtitle.property("type").toString() == "file") {
         QString filename = subtitle.name();
 
-        if (!filename.startsWith("file://"))
-            filename.prepend("file://");
+        if (!filename.startsWith(QLatin1String("file://")))
+            filename.prepend(QLatin1String("file://"));
         // It's not possible to change the suburi when the pipeline is PLAYING mainly
         // because the pipeline has not been built with the subtitle element. A workaround
         // consists to restart the pipeline and set the suburi property (totem does exactly the same thing)
@@ -775,7 +839,7 @@ void MediaObject::changeTitle(const QString &format, int title)
     if (!titleFormat)
         return;
 
-    m_backend->logMessage(QString("changeTitle %0 %1").arg(format).arg(title), Backend::Info, this);
+    debug() << Q_FUNC_INFO << format << title;
     if (gst_element_seek_simple(m_pipeline->element(), titleFormat, GST_SEEK_FLAG_FLUSH, title - 1)) {
         m_currentTitle = title;
         emit titleChanged(title);
@@ -785,13 +849,13 @@ void MediaObject::changeTitle(const QString &format, int title)
 
 void MediaObject::logWarning(const QString &msg)
 {
-    m_backend->logMessage(msg, Backend::Warning);
+    warning() << msg;
 }
 
 void MediaObject::handleBuffering(int percent)
 {
     Q_ASSERT(0);
-    m_backend->logMessage(QString("Stream buffering %0").arg(percent), Backend::Debug, this);
+    debug() << Q_FUNC_INFO << percent;
     if (m_state != Phonon::BufferingState)
         emit stateChanged(m_state, Phonon::BufferingState);
     else if (percent == 100)
@@ -849,7 +913,8 @@ void MediaObject::handleAboutToFinish()
     debug() << "About to finish";
     m_aboutToFinishLock.lock();
     m_handlingAboutToFinish = true;
-    emit aboutToFinish();
+    if (!m_waitingForNextSource)
+        emit aboutToFinish();
     // As our signal gets emitted queued we need to wait here until either a
     // new source or a timeout is reached.
     // If we got a new source in time -> hooray + gapless
@@ -894,6 +959,4 @@ void MediaObject::handleAboutToFinish()
 } // ns Gstreamer
 } // ns Phonon
 
-QT_END_NAMESPACE
-
 #include "moc_mediaobject.cpp"
