@@ -1,7 +1,7 @@
 /*  This file is part of the KDE project.
 
     Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-    Copyright (C) 2011 Trever Fischer <tdfischer@fedoraproject.org>
+    Copyright (C) 2011 Torrie Fischer <tdfischer@kde.org>
     Copyright (C) 2011 Harald Sitter <sitter@kde.org>
 
     This library is free software: you can redistribute it and/or modify
@@ -18,9 +18,10 @@
 */
 
 #include "pipeline.h"
-#include "debug.h"
+
 #include "mediaobject.h"
 #include "backend.h"
+#include "debug.h"
 #include "plugininstaller.h"
 #include "streamreader.h"
 #include "gsthelper.h"
@@ -29,10 +30,8 @@
 #include <gst/app/gstappsrc.h>
 #include <QtCore/QCoreApplication>
 #include <QtCore/QMutexLocker>
-#include "debug.h"
-#define MAX_QUEUE_TIME 20 * GST_SECOND
 
-QT_BEGIN_NAMESPACE
+#define MAX_QUEUE_TIME 20 * GST_SECOND
 namespace Phonon
 {
 namespace Gstreamer
@@ -49,10 +48,10 @@ Pipeline::Pipeline(QObject *parent)
 {
     qRegisterMetaType<GstState>("GstState");
     m_pipeline = GST_PIPELINE(gst_element_factory_make("playbin2", NULL));
-    gst_object_ref(m_pipeline);
-    gst_object_sink(m_pipeline);
+    gst_object_ref_sink (m_pipeline);
     g_signal_connect(m_pipeline, "video-changed", G_CALLBACK(cb_videoChanged), this);
     g_signal_connect(m_pipeline, "text-tags-changed", G_CALLBACK(cb_textTagsChanged), this);
+    g_signal_connect(m_pipeline, "audio-tags-changed", G_CALLBACK(cb_audioTagsChanged), this);
     g_signal_connect(m_pipeline, "notify::source", G_CALLBACK(cb_setupSource), this);
     g_signal_connect(m_pipeline, "about-to-finish", G_CALLBACK(cb_aboutToFinish), this);
 
@@ -118,7 +117,7 @@ Pipeline::Pipeline(QObject *parent)
         g_object_set(G_OBJECT(m_audioPipe), "max-size-bytes", 0, NULL);
     }
 
-    connect(m_installer, SIGNAL(failure(const QString&)), this, SLOT(pluginInstallFailure(const QString&)));
+    connect(m_installer, SIGNAL(failure(QString)), this, SLOT(pluginInstallFailure(QString)));
     connect(m_installer, SIGNAL(started()), this, SLOT(pluginInstallStarted()));
     connect(m_installer, SIGNAL(success()), this, SLOT(pluginInstallComplete()));
 }
@@ -151,12 +150,6 @@ void Pipeline::setSource(const Phonon::MediaSource &source, bool reset)
     m_resumeAfterInstall = false;
     m_isHttpUrl = false;
     m_metaData.clear();
-    if (m_reader) {
-        m_reader->stop();
-        // Because libphonon stream stuff likes to fail connection assert
-        delete m_reader;
-        m_reader = 0;
-    }
 
     debug() << "New source:" << source.mrl();
     QByteArray gstUri;
@@ -212,6 +205,7 @@ void Pipeline::setSource(const Phonon::MediaSource &source, bool reset)
         gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_READY);
     }
 
+    debug() << "uri" << gstUri;
     g_object_set(m_pipeline, "uri", gstUri.constData(), NULL);
 
     if (reset && oldState > GST_STATE_READY) {
@@ -221,8 +215,10 @@ void Pipeline::setSource(const Phonon::MediaSource &source, bool reset)
 
 Pipeline::~Pipeline()
 {
+    g_signal_handlers_disconnect_by_data(m_pipeline, this);
     gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_NULL);
     gst_object_unref(m_pipeline);
+    m_pipeline = 0;
 }
 
 GstElement *Pipeline::element() const
@@ -236,8 +232,10 @@ GstStateChangeReturn Pipeline::setState(GstState state)
     m_resumeAfterInstall = true;
     debug() << "Transitioning to state" << GstHelper::stateName(state);
 
-    if (state == GST_STATE_READY && m_reader)
+    if (state == GST_STATE_READY && m_reader) {
+        debug() << "forcing stop as we are in ready state and have a reader...";
         m_reader->stop();
+    }
 
     return gst_element_set_state(GST_ELEMENT(m_pipeline), state);
 }
@@ -246,7 +244,7 @@ void Pipeline::writeToDot(MediaObject *media, const QString &type)
 {
     GstBin *bin = GST_BIN(m_pipeline);
     if (media)
-        media->backend()->logMessage(QString("Dumping %0.dot").arg(type), Backend::Debug, media);
+        debug() << media << "Dumping" << QString("%0.dot").arg(type);
     else {
         debug() << type;
     }
@@ -401,6 +399,12 @@ void Pipeline::cb_textTagsChanged(GstElement *playbin, gint stream, gpointer dat
     emit that->textTagChanged(stream);
 }
 
+void Pipeline::cb_audioTagsChanged(GstElement *playbin, gint stream, gpointer data)
+{
+    Pipeline *that = static_cast<Pipeline *>(data);
+    emit that->audioTagChanged(stream);
+}
+
 bool Pipeline::videoIsAvailable() const
 {
     gint videoCount;
@@ -551,7 +555,7 @@ void foreach_tag_function(const GstTagList *list, const gchar *tag, gpointer use
         break;
 
     default:
-        //debug("Unsupported tag type: %s", g_type_name(type));
+        //debug() << "Unsupported tag type:" << g_type_name(type);
         break;
     }
 
@@ -622,12 +626,7 @@ gboolean Pipeline::cb_tag(GstBus *bus, GstMessage *msg, gpointer data)
             that->m_metaData.insert("MUSICBRAINZ_DISCID", newTags.value("MUSICBRAINZ-DISCID"));
         }
 
-        // For radio streams, if we get a metadata update where the title changes, we assume everything else is invalid.
-        // If we don't already have a title, we don't do anything since we're actually just appending new data into that.
-        if (that->m_isStream && oldMap.contains("TITLE") && that->m_metaData.value("TITLE") != oldMap.value("TITLE")) {
-            that->m_metaData.clear();
-        }
-
+        //debug() << this << "Meta tags found";
         if (oldMap != that->m_metaData) {
             // This is a bit of a hack to ensure that stream metadata is
             // returned. We get as much as we can from the Shoutcast server's
@@ -764,22 +763,20 @@ bool Pipeline::seekToMSec(qint64 time)
 
 bool Pipeline::isSeekable() const
 {
+    gboolean seekable = 0;
     GstQuery *query;
     gboolean result;
     gint64 start, stop;
     query = gst_query_new_seeking(GST_FORMAT_TIME);
     result = gst_element_query (GST_ELEMENT(m_pipeline), query);
     if (result) {
-        gboolean seekable;
         GstFormat format;
         gst_query_parse_seeking(query, &format, &seekable, &start, &stop);
-        gst_query_unref (query);
-        return seekable;
     } else {
         //TODO: Log failure
     }
     gst_query_unref (query);
-    return false;
+    return seekable;
 }
 
 Phonon::State Pipeline::phononState() const
@@ -805,8 +802,11 @@ static void cb_feedAppSrc(GstAppSrc *appSrc, guint buffsize, gpointer data)
     DEBUG_BLOCK;
     StreamReader *reader = static_cast<StreamReader*>(data);
     GstBuffer *buf = gst_buffer_new_and_alloc(buffsize);
+#warning ret not used!!! WHOOPWHOOPWHOOP
     reader->read(reader->currentPos(), buffsize, (char*)GST_BUFFER_DATA(buf));
     gst_app_src_push_buffer(appSrc, buf);
+    if (GST_BUFFER_SIZE(buf) > 0 && reader->atEnd())
+        gst_app_src_end_of_stream(appSrc);
 }
 
 static void cb_seekAppSrc(GstAppSrc *appSrc, guint64 pos, gpointer data)
@@ -822,13 +822,24 @@ void Pipeline::cb_setupSource(GstElement *playbin, GParamSpec *param, gpointer d
     Q_UNUSED(playbin);
     Q_UNUSED(param);
     DEBUG_BLOCK;
+
     GstElement *phononSrc;
     Pipeline *that = static_cast<Pipeline*>(data);
-    gst_object_ref(that->m_pipeline);
+    Q_ASSERT(that->m_pipeline);
+    Q_ASSERT(G_IS_OBJECT(that->m_pipeline));
     g_object_get(that->m_pipeline, "source", &phononSrc, NULL);
+
+    if (that->m_reader) {
+        // Because libphonon stream stuff likes to fail connection asserts
+        // we force a complete reset.
+        that->m_reader->stop();
+        delete that->m_reader;
+        that->m_reader = 0;
+    }
+
     if (that->m_isStream) {
-        if (!that->m_reader)
-            that->m_reader = new StreamReader(that->m_currentSource, that);
+        that->m_reader = new StreamReader(that->m_currentSource, that);
+        that->m_reader->start();
         if (that->m_reader->streamSize() > 0)
             g_object_set(phononSrc, "size", that->m_reader->streamSize(), NULL);
         int streamType = 0;
@@ -840,7 +851,6 @@ void Pipeline::cb_setupSource(GstElement *playbin, GParamSpec *param, gpointer d
         g_object_set(phononSrc, "block", TRUE, NULL);
         g_signal_connect(phononSrc, "need-data", G_CALLBACK(cb_feedAppSrc), that->m_reader);
         g_signal_connect(phononSrc, "seek-data", G_CALLBACK(cb_seekAppSrc), that->m_reader);
-        that->m_reader->start();
     } else {
         if (that->currentSource().type() == MediaSource::Url
                 && that->currentSource().mrl().scheme().startsWith(QLatin1String("http"))
@@ -856,7 +866,6 @@ void Pipeline::cb_setupSource(GstElement *playbin, GParamSpec *param, gpointer d
             g_object_set(phononSrc, "device", that->currentSource().deviceName().toUtf8().constData(), NULL);
         }
     }
-    gst_object_unref(that->m_pipeline);
 }
 
 void Pipeline::cb_aboutToFinish(GstElement *appSrc, gpointer data)
@@ -890,7 +899,7 @@ QByteArray Pipeline::captureDeviceURI(const MediaSource &source) const
     if (source.videoCaptureDevice().isValid()) {
         DeviceAccessList devList = source.videoCaptureDevice().property("deviceAccessList").value<Phonon::DeviceAccessList>();
         QString devPath;
-        foreach (DeviceAccess dev, devList) {
+        foreach (const DeviceAccess &dev, devList) {
             if (dev.first == "v4l2") {
                 return QString("v4l2://%0").arg(dev.second).toUtf8();
             }
